==> 0.4.2&file=in_ffsox-2a-0.4.2-src.tar.gz: in_ffsox-2a-0.4.2/in_ffsox/video.c <==
/*
 * video.c
 * Copyright (C) 2010 - 2013 Peter Belkner <pbelkner@snafu.de>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "plugin.h"

video_t *video_init(video_t *v, decoder_t *dec, In_Module *mod)
{
  v->dec=dec;
  v->mod=mod;

  if (NULL==(v->frame=av_frame_alloc()))
    goto error;

  //avcodec_get_frame_defaults(v->frame);
  timer_init(&v->timer);

  if (NULL==queue_init(&v->q,dec,"Video",dec->ps->video.queue_size))
    goto error;

  if (NULL!=mod&&NULL==screen_init(&v->s,mod))
    goto error;

  return v;
error:
  video_cleanup(v);

  return NULL;
}

video_t *video_cleanup(video_t *v)
{
  screen_cleanup(&v->s);
  queue_cleanup(&v->q);
  timer_cleanup(&v->timer);

  if (NULL!=v->frame)
    av_free(v->frame);

  return v;
}

video_t *video_realloc(video_t *v, track_t *t)
{
  char filter_descr[128];
#if defined (PREFS_YADIF) // [
  prefs_t *ps=v->dec->ps;
#endif // ]
  ffi_t *ffi=&t->ffi;
  ffi_st_t *vst=ffi->vst;
  AVStream *st=vst->st;
  AVCodecContext *dc=vst->dc;
  char *crop;
  char *ar;
  double num,denom;
  double ar1,ar2;

  v->st=vst;
  dc=vst->dc;

  if (NULL!=(crop=track_get_tag(t,"video_crop"))
      &&4==sscanf(crop,"%d:%d:%d:%d",&v->width,&v->height,&v->x,&v->y)) {

    // x
    if (v->width<v->x)
      v->x=v->width;

    // y
    if (v->height<v->y)
      v->y=v->height;

    // width
    if (0==v->width)
      v->width=dc->width;

    if (dc->width<v->x+v->width)
      v->width=dc->width-v->x;

    if (v->width<0)
      v->width=0;

    // height
    if (0==v->height)
      v->height=dc->height;

    if (dc->height<v->y+v->height)
      v->height=dc->height-v->y;

    if (v->height<0)
      v->height=0;

    v->crop=1;
  }
  else {
    v->width=dc->width;
    v->height=dc->height;
    v->x=0;
    v->y=0;
    v->crop=0;
  }

  ar1=st->sample_aspect_ratio.num?av_q2d(st->sample_aspect_ratio)
      :dc->sample_aspect_ratio.num?av_q2d(dc->sample_aspect_ratio)
      :0.0
      ;

  ar2=ar1<=0.0?1.0:ar1;

  if (NULL!=(ar=track_get_tag(t,"video_aspect_ratio"))
      &&2==sscanf(ar,"%lf:%lf",&num,&denom)) {

    ar2*=(double)num/denom*v->height/v->width;
  }

  screen_open(&v->s,v->width,v->height,ar2);
#if defined (PREFS_YADIF) // [
  if (!ps->video.yadif.disabled) {
    sprintf(filter_descr,"yadif=%d:%d:%d",
        ps->video.yadif.mode,
        ps->video.yadif.parity,
        ps->video.yadif.deint
    );
#else // ] [
    sprintf(filter_descr,"yadif=%d:%d:%d",
        eYadifModeSendFrame,
        eYadifParityAuto,
#if 1 // [
        eYadifDeintInterlaced
#else // ] [
        eYadifDeintAll
#endif // ]
    );
#endif // ]

DPUTS("-------------------------------------------------------------\n");
DPRINTF("filter_descr=\"%s\"\n",filter_descr);
DPUTS("-------------------------------------------------------------\n");
    if (video_filter_init(&v->vf,ffi->ic,vst->index,dc,filter_descr)<0) {
      DMESSAGE("initializing video filter");
      goto error;
    }
#if defined (PREFS_YADIF) // [
  }
#endif // ]

  v->time_base=1000.0*av_q2d(st->time_base);

  return v;
error:
  return NULL;
}

void video_reset(video_t *v)
{
  prefs_t *ps=v->dec->ps;

  //avcodec_get_frame_defaults(v->frame);
  v->timer.state=STATE_RUN;
  v->next_pts=0;
  queue_reset(&v->q,ps->video.queue_size);
}

video_t *video_run(video_t *v)
{
  v->hThread=CreateThread(
    NULL,   // _In_opt_   LPSECURITY_ATTRIBUTES lpThreadAttributes,
    0,      // _In_       SIZE_T dwStackSize,
    video_thread,
            // _In_       LPTHREAD_START_ROUTINE lpStartAddress,
    v,      // _In_opt_   LPVOID lpParameter,
    0,      // _In_       DWORD dwCreationFlags,
    NULL    // _Out_opt_  LPDWORD lpThreadId
  );

  return NULL==v->hThread?NULL:v;
}

void video_wait(video_t *v)
{
  if (NULL!=v->hThread) {
    WaitForSingleObject(v->hThread,INFINITE);
    CloseHandle(v->hThread);
    v->hThread=NULL;
  }
}

void video_stop(video_t *v)
{
  screen_done(&v->s);
  video_filter_cleanup(&v->vf);
  v->st=NULL;
}

void video_kill(video_t *v)
{
  timer_kill(&v->timer);
}

#if 1 // {
#define video_drift(v) \
  (round((v)->time_base*(v)->frame->pts-decoder_get_time((v)->dec,(v)->mod)))
#else // } {
#define video_drift(v) \
  (round((v)->time_base*(v)->frame->pts-(v)->mod->GetOutputTime()))
#endif // }

static void video_display_frame(video_t *v, AVFrame *frame)
{
  decoder_t *dec=v->dec;
  prefs_t *ps=dec->ps;
  ffi_st_t *vst=v->st;
  AVCodecContext *dc=vst->dc;
  AVPicture crop;
  AVPicture *pict;
  double drift;

  pict=(AVPicture *)frame;

  if (v->crop&&(0<v->x||0<v->y)) {
    av_picture_crop(&crop,pict,dc->pix_fmt,v->y,v->x);
    pict=&crop;
  }

#if 0 // [
  frame->pts=av_frame_get_best_effort_timestamp(frame);
#endif // ]
  drift=video_drift(v);

  if (ps->video.max_drift<=drift) {
    DPRINTF("DRIFT 1: %.0f\n",drift);
    frame->pts=v->next_pts;
    drift=video_drift(v);
  }

  if (ps->video.min_drift<=drift&&drift<ps->video.max_drift) {
    if (0.0<drift) {
      //DPRINTF("DRIFT 2: %.0f\n",drift);
      timer_sleep(&v->timer,drift);
    }
    else
      DPRINTF("DRIFT 3: %.0f\n",drift);

    if (STATE_KILL!=v->timer.state)
      screen_draw(&v->s,pict);

    v->next_pts=frame->pts+frame->pkt_duration;
  }
  else
    DPRINTF("DRIFT 4: %.0f\n",drift);

  //avcodec_get_frame_defaults(frame);
}

void video_display(video_t *v, AVPacket *pkt, int *got_frame)
{
  ffi_st_t *vst=v->st;
  AVCodecContext *dc=vst->dc;
  int len;
  int ret;

  if ((len=avcodec_decode_video2(dc,v->frame,got_frame,pkt))<0)
    goto cleanup;

  pkt->size-=len;
  pkt->data+=len;

  if (*got_frame) {
    if (v->vf.frame) {
      // Adapted from main() @ "<ffmpeg>/doc/examples/filering_video.c" [
      v->frame->pts=v->frame->best_effort_timestamp;

      // push the decoded frame into the filtergraph
      ret=video_filter_push_frame(&v->vf,v->frame);

      if (ret<0) {
        DVMESSAGE("av_buffersrc_add_frame_flags() failed with ret=%d (%s)",
            ret,av_err2str(ret));
        goto cleanup;
      }

      // pull filtered frames from the filtergraph
      for (;;) {
        ret=video_filter_pull_frame(&v->vf);

        if (ret == AVERROR(EAGAIN) || ret == AVERROR_EOF)
            break;

        if (ret<0)
            goto cleanup;

        //display_frame(filt_frame, buffersink_ctx->inputs[0]->time_base);
        video_display_frame(v,v->vf.frame);
        av_frame_unref(v->vf.frame);
      }
    // ]
    }
    else
      video_display_frame(v,v->frame);

    av_frame_unref(v->frame);
  }
cleanup:
  return;
}

#define PKT2
DWORD WINAPI video_thread(LPVOID lpParam)
{
  video_t *v=lpParam;
#if defined (PKT2) // {
  AVPacket pkt1,pkt2;
#else // } {
  AVPacket pkt;
#endif // }
  int got_frame;

#if defined (PKT2) // {
  memset(&pkt1,0,sizeof pkt1);
  memset(&pkt2,0,sizeof pkt2);

  for (;;) {
    if (pkt2.size<=0) {
      av_free_packet(&pkt1);
      memset(&pkt1,0,sizeof pkt1);

      if (0!=queue_read(&v->q,&pkt1))
        break;

      pkt2=pkt1;
    }

    video_display(v,&pkt2,&got_frame);
  }
#else // } {
  while (0==queue_read(&v->q,&pkt)) {
    video_display(v,&pkt,&got_frame);
    av_free_packet(&pkt);
  }
#endif // }

  if (STATE_KILL!=v->timer.state) {
    // flush cached frames
#if defined (PKT2) // {
    pkt2.data=NULL;
    pkt2.size=0;
#else // } {
    pkt.data=NULL;
    pkt.size=0;
#endif // }

    do {
#if defined (PKT2) // {
      video_display(v,&pkt2,&got_frame);
#else // } {
      video_display(v,&pkt,&got_frame);
#endif // }
    } while (got_frame);
  }

  return 0;
}

==> 0.4.3&file=in_ffsox-2a-0.4.3-src.tar.gz: in_ffsox-2a-0.4.3/in_ffsox/video.c <==
/*
 * video.c
 * Copyright (C) 2010 - 2013 Peter Belkner <pbelkner@snafu.de>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "plugin.h"

video_t *video_init(video_t *v, decoder_t *dec, In_Module *mod)
{
  v->dec=dec;
  v->mod=mod;

  if (NULL==(v->frame=av_frame_alloc()))
    goto error;

  //avcodec_get_frame_defaults(v->frame);
  timer_init(&v->timer);

  if (NULL==queue_init(&v->q,dec,"Video",dec->ps->video.queue_size))
    goto error;

  if (NULL!=mod&&NULL==screen_init(&v->s,mod))
    goto error;

  return v;
error:
  video_cleanup(v);

  return NULL;
}

video_t *video_cleanup(video_t *v)
{
  screen_cleanup(&v->s);
  queue_cleanup(&v->q);
  timer_cleanup(&v->timer);

  if (NULL!=v->frame)
    av_free(v->frame);

  return v;
}

video_t *video_realloc(video_t *v, track_t *t)
{
  char filter_descr[128];
  char *wp=filter_descr;
#if defined (PREFS_YADIF) // [
  prefs_t *ps=v->dec->ps;
#endif // ]
  ffi_t *ffi=&t->ffi;
  ffi_st_t *vst=ffi->vst;
  AVStream *st=vst->st;
  AVCodecContext *dc=vst->dc1;
  char *crop;
  char *ar;
  double num,denom;
  double ar1,ar2;

  *wp=0;

  v->st=vst;
#if 0 // [
  dc=vst->dc;
#endif // ]

  if (NULL!=(crop=track_get_tag(t,"video_crop"))
      &&4==sscanf(crop,"%d:%d:%d:%d",&v->width,&v->height,&v->x,&v->y)) {

    // x
    if (v->width<v->x)
      v->x=v->width;

    // y
    if (v->height<v->y)
      v->y=v->height;

    // width
    if (0==v->width)
      v->width=dc->width;

    if (dc->width<v->x+v->width)
      v->width=dc->width-v->x;

    if (v->width<0)
      v->width=0;

    // height
    if (0==v->height)
      v->height=dc->height;

    if (dc->height<v->y+v->height)
      v->height=dc->height-v->y;

    if (v->height<0)
      v->height=0;

    v->crop=1;
  }
  else {
    v->width=dc->width;
    v->height=dc->height;
    v->x=0;
    v->y=0;
    v->crop=0;
  }

  ar1=st->sample_aspect_ratio.num?av_q2d(st->sample_aspect_ratio)
      :dc->sample_aspect_ratio.num?av_q2d(dc->sample_aspect_ratio)
      :0.0
      ;

  ar2=ar1<=0.0?1.0:ar1;

  if (NULL!=(ar=track_get_tag(t,"video_aspect_ratio"))
      &&2==sscanf(ar,"%lf:%lf",&num,&denom)) {

    ar2*=(double)num/denom*v->height/v->width;
  }

  screen_open(&v->s,v->width,v->height,ar2);
#if defined (PREFS_YADIF) // [
  if (!ps->video.yadif.disabled) {
    sprintf(wp,"yadif=%d:%d:%d",
        ps->video.yadif.mode,
        ps->video.yadif.parity,
        ps->video.yadif.deint
    );
#else // ] [
    sprintf(wp,"yadif=%d:%d:%d",
        eYadifModeSendFrame,
        eYadifParityAuto,
#if 1 // [
        eYadifDeintInterlaced
#else // ] [
        eYadifDeintAll
#endif // ]
    );
#endif // ]

  wp+=strlen(wp);

#if defined (FFSOX_DEPRECATED_PICTURE_CROP) // [
  if (v->crop) {
    if (filter_descr<wp) {
      *wp++=',';
      *wp=0;
    }

    sprintf(wp,"crop=%d:%d:%d:%d",v->width,v->height,v->x,v->y);
  }
#endif // ]

DPUTS("-------------------------------------------------------------\n");
DPRINTF("filter_descr=\"%s\"\n",filter_descr);
DPUTS("-------------------------------------------------------------\n");
    if (video_filter_init(&v->vf,ffi->ic,vst->index,dc,filter_descr)<0) {
      DMESSAGE("initializing video filter");
      goto error;
    }
#if defined (PREFS_YADIF) // [
  }
#endif // ]

  v->time_base=1000.0*av_q2d(st->time_base);

  return v;
error:
  return NULL;
}

void video_reset(video_t *v)
{
  prefs_t *ps=v->dec->ps;

  //avcodec_get_frame_defaults(v->frame);
  v->timer.state=STATE_RUN;
  v->next_pts=0;
  queue_reset(&v->q,ps->video.queue_size);
}

video_t *video_run(video_t *v)
{
  v->hThread=CreateThread(
    NULL,   // _In_opt_   LPSECURITY_ATTRIBUTES lpThreadAttributes,
    0,      // _In_       SIZE_T dwStackSize,
    video_thread,
            // _In_       LPTHREAD_START_ROUTINE lpStartAddress,
    v,      // _In_opt_   LPVOID lpParameter,
    0,      // _In_       DWORD dwCreationFlags,
    NULL    // _Out_opt_  LPDWORD lpThreadId
  );

  return NULL==v->hThread?NULL:v;
}

void video_wait(video_t *v)
{
  if (NULL!=v->hThread) {
    WaitForSingleObject(v->hThread,INFINITE);
    CloseHandle(v->hThread);
    v->hThread=NULL;
  }
}

void video_stop(video_t *v)
{
  screen_done(&v->s);
  video_filter_cleanup(&v->vf);
  v->st=NULL;
}

void video_kill(video_t *v)
{
  timer_kill(&v->timer);
}

#if 1 // {
#define video_drift(v) \
  (round((v)->time_base*(v)->frame->pts-decoder_get_time((v)->dec,(v)->mod)))
#else // } {
#define video_drift(v) \
  (round((v)->time_base*(v)->frame->pts-(v)->mod->GetOutputTime()))
#endif // }

static void video_display_frame(video_t *v, AVFrame *frame)
{
  decoder_t *dec=v->dec;
  prefs_t *ps=dec->ps;
  double drift;
#if ! defined (FFSOX_DEPRECATED_PICTURE_CROP) // [
  ffi_st_t *vst=v->st;
  AVCodecContext *dc=vst->dc1;
  AVPicture *pict=(AVPicture *)frame;
  AVPicture crop;

  if (v->crop&&(0<v->x||0<v->y)) {
    av_picture_crop(&crop,pict,dc->pix_fmt,v->y,v->x);
    pict=&crop;
  }
#endif // ]

#if 0 // [
  frame->pts=av_frame_get_best_effort_timestamp(frame);
#endif // ]
  drift=video_drift(v);

  if (ps->video.max_drift<=drift) {
    DPRINTF("DRIFT 1: %.0f\n",drift);
    frame->pts=v->next_pts;
    drift=video_drift(v);
  }

  if (ps->video.min_drift<=drift&&drift<ps->video.max_drift) {
    if (0.0<drift) {
      //DPRINTF("DRIFT 2: %.0f\n",drift);
      timer_sleep(&v->timer,drift);
    }
    else
      DPRINTF("DRIFT 3: %.0f\n",drift);

    if (STATE_KILL!=v->timer.state)
#if defined (FFSOX_DEPRECATED_PICTURE_CROP) // [
      screen_draw(&v->s,frame);
#else // ] [
      screen_draw(&v->s,pict);
#endif // ]

    v->next_pts=frame->pts+frame->pkt_duration;
  }
  else
    DPRINTF("DRIFT 4: %.0f\n",drift);

  //avcodec_get_frame_defaults(frame);
}

#if defined (FFSOX_DEPRECATED_DECODE_VIDEO2) // [
// https://blogs.gentoo.org/lu_zero/2016/03/29/new-avcodec-api/
void video_display(video_t *v, AVPacket *pkt)
{
  ffi_st_t *vst=v->st;
  AVCodecContext *dc=vst->dc1;
  int fferr;

  // If there's a valid packet we send/push it to the decoder. Otherwise
  // we just need to flush the decoder.
  if (pkt&&pkt->size) {
#if 0 // [
  send:
#endif // ]
    fferr=avcodec_send_packet(dc,pkt);

    if (fferr<0) {
      switch (fferr) {
      case AVERROR(EAGAIN):
        // According to Luca Barbato's Gentoo Blog: AVERROR(EAGAIN), that
        // signals that the internal input buffer is full.
        // We should never go here because we always flush the input buffer
        // as soon as the next frame is available.
        DMESSAGE("avcodec_send_packet() failed with AVERROR(EAGAIN)");
#if 0 // [
        Sleep(10);
        goto send;
#else // ] [
        goto cleanup;
#endif // ]
      default:
        DVMESSAGE("avcodec_send_packet() failed with %d (%s)",
            fferr,av_err2str(fferr));
        goto cleanup;
      }
    }
  }

  // As long as there are frames availabe receive/pull them from the decoder.
  for (;;) {
    fferr=avcodec_receive_frame(dc,v->frame);

    if (fferr<0) {
      switch (fferr) {
      case AVERROR(EAGAIN):
        // No error at all. This particular code just seem to signal that
        // the decoder is flushed.
#if 0 // [
        DMESSAGE("avcodec_receive_frame() failed with AVERROR(EAGAIN)");
#endif // ]
        goto cleanup;
      default:
        DVMESSAGE("avcodec_receive_frame() failed with %x (%s)",
            fferr,av_err2str(fferr));
        goto cleanup;
      }
    }
    else if (v->vf.frame) {
      // Adapted from main() @ "<ffmpeg>/doc/examples/filering_video.c" [
      v->frame->pts=v->frame->best_effort_timestamp;

      // push the decoded frame into the filtergraph
      fferr=video_filter_push_frame(&v->vf,v->frame);

      if (fferr<0) {
        DVMESSAGE("av_buffersrc_add_frame_flags() failed with %d (%s)",
            fferr,av_err2str(fferr));
        goto cleanup;
      }

      // pull filtered frames from the filtergraph
      for (;;) {
        fferr=video_filter_pull_frame(&v->vf);

        if (fferr==AVERROR(EAGAIN)||fferr==AVERROR_EOF)
          break;

        if (fferr<0)
          goto cleanup;

        //display_frame(filt_frame, buffersink_ctx->inputs[0]->time_base);
        video_display_frame(v,v->vf.frame);
        av_frame_unref(v->vf.frame);
      }
      // ]
    }
    else
      video_display_frame(v,v->frame);

    av_frame_unref(v->frame);
  }
cleanup:
  return;
}
#else // ] [
void video_display(video_t *v, AVPacket *pkt, int *got_frame)
{
  ffi_st_t *vst=v->st;
  AVCodecContext *dc=vst->dc1;
  int len;
  int ret;

  if ((len=avcodec_decode_video2(dc,v->frame,got_frame,pkt))<0)
    goto cleanup;

  pkt->size-=len;
  pkt->data+=len;

  if (*got_frame) {
    if (v->vf.frame) {
      // Adapted from main() @ "<ffmpeg>/doc/examples/filering_video.c" [
      v->frame->pts=v->frame->best_effort_timestamp;

      // push the decoded frame into the filtergraph
      ret=video_filter_push_frame(&v->vf,v->frame);

      if (ret<0) {
        DVMESSAGE("av_buffersrc_add_frame_flags() failed with ret=%d (%s)",
            ret,av_err2str(ret));
        goto cleanup;
      }

      // pull filtered frames from the filtergraph
      for (;;) {
        ret=video_filter_pull_frame(&v->vf);

        if (ret == AVERROR(EAGAIN) || ret == AVERROR_EOF)
          break;

        if (ret<0)
          goto cleanup;

        //display_frame(filt_frame, buffersink_ctx->inputs[0]->time_base);
        video_display_frame(v,v->vf.frame);
        av_frame_unref(v->vf.frame);
      }
      // ]
    }
    else
      video_display_frame(v,v->frame);

    av_frame_unref(v->frame);
  }
cleanup:
  return;
}
#endif // ]

#if ! defined (FFSOX_DEPRECATED_DECODE_VIDEO2) // [
#define PKT2
#endif // ]
DWORD WINAPI video_thread(LPVOID lpParam)
{
  video_t *v=lpParam;
#if defined (PKT2) // {
  AVPacket pkt1,pkt2;
#else // } {
  AVPacket pkt;
#endif // }
#if ! defined (FFSOX_DEPRECATED_DECODE_VIDEO2) // [
  int got_frame;
#endif // ]

#if defined (PKT2) // {
  memset(&pkt1,0,sizeof pkt1);
  memset(&pkt2,0,sizeof pkt2);

  for (;;) {
    if (pkt2.size<=0) {
#if defined (FFSOX_DEPRECATED_FREE_PACKET) // [
      av_packet_unref(&pkt1);
#else // ] [
      av_free_packet(&pkt1);
#endif // ]
      memset(&pkt1,0,sizeof pkt1);

      if (0!=queue_read(&v->q,&pkt1))
        break;

      pkt2=pkt1;
    }

    video_display(v,&pkt2,&got_frame);
  }
#else // } {
  while (0==queue_read(&v->q,&pkt)) {
#if defined (FFSOX_DEPRECATED_DECODE_VIDEO2) // [
    video_display(v,&pkt);
#else // ] [
    video_display(v,&pkt,&got_frame);
#endif // ]
#if defined (FFSOX_DEPRECATED_FREE_PACKET) // [
    av_packet_unref(&pkt);
#else // ] [
    av_free_packet(&pkt);
#endif // ]
  }
#endif // }

  if (STATE_KILL!=v->timer.state) {
    // flush cached frames
#if defined (PKT2) // {
    pkt2.data=NULL;
    pkt2.size=0;
#else // } {
    pkt.data=NULL;
    pkt.size=0;
#endif // }

#if defined (FFSOX_DEPRECATED_DECODE_VIDEO2) // [
    video_display(v,NULL);
#else // ] [
    do {
#if defined (PKT2) // {
      video_display(v,&pkt2,&got_frame);
#else // } {
      video_display(v,&pkt,&got_frame);
#endif // }
    } while (got_frame);
#endif // ]
  }

  return 0;
}
